require 'uri'

module Printos

  API_BASE_PATH = 'api/aaa/v3'

  module AaaOrganization

    ORGANIZATIONS_PATH = "#{API_BASE_PATH}/organizations"

    def organization
      get_resource(ORGANIZATIONS_PATH)
    end

    def update_organization(payload)
      put_resource(ORGANIZATIONS_PATH, payload)
    end
  end

  module AaaOrganizationSite

    ORGANIZATION_SITES_PATH = "#{API_BASE_PATH}/organizations/sites"

    def sites
      get_resource(ORGANIZATION_SITES_PATH)
    end

    def site(resource_id)
      get_resource("#{ORGANIZATION_SITES_PATH}/#{resource_id}")
    end

    def create_site(payload)
      post_resource(ORGANIZATION_SITES_PATH, payload)
    end

    def update_site(resource_id, payload)
      put_resource("#{ORGANIZATION_SITES_PATH}/#{resource_id}", payload)
    end
  end

  module AaaOrganizationAddress

    ORGANIZATION_ADDRESSES_PATH = "#{API_BASE_PATH}/organizations/addresses"

    def addresses
      get_resource(ORGANIZATION_ADDRESSES_PATH)
    end

    def address(resource_id)
      get_resource("#{ORGANIZATION_ADDRESSES_PATH}/#{resource_id}")
    end

    def create_address(payload)
      post_resource(ORGANIZATION_ADDRESSES_PATH, payload)
    end

    def update_address(resource_id, payload)
      put_resource("#{ORGANIZATION_ADDRESSES_PATH}/#{resource_id}", payload)
    end
  end

  class AaaService < BaseService

    extend AaaOrganization
    extend AaaOrganizationSite
    extend AaaOrganizationAddress

    def self.user
      get_resource("#{API_BASE_PATH}/users")
    end

    def self.device(resource_id)
      get_resource("#{API_BASE_PATH}/organizations/devices/#{resource_id}")
    end

    def self.devices
      get_resource("#{API_BASE_PATH}/organizations/devices/v2")
    end

    def self.admin_context
      get_resource("#{API_BASE_PATH}/admin/context")
    end

    def self.admin_device(resource_id)
      get_resource("#{API_BASE_PATH}/admin/devices/#{resource_id}")
    end

    def self.admin_device_gbu_organizations(offset, limit, device_gbu_filter, device_type_filter)
      uri = URI.join("#{API_BASE_PATH}/admin/devicegbu/organizations", org_id)
      query_params = URI.decode_www_form(uri.query || '')
      query_params << ['offset', offset] unless offset.blank?
      query_params << ['limit', limit] unless limit.blank?
      query_params << ['deviceType', device_type_filter] unless device_type_filter.blank?
      query_params << ['deviceGBU', device_gbu_filter] unless device_gbu_filter.blank?
      uri.query = URI.encode_www_form(query_params)

      get_resource(uri.to_s)
    end

    def self.admin_organization(org_id)
      get_resource("#{API_BASE_PATH}/admin/organizations/#{org_id}")
    end

    def self.admin_organizations(offset, limit, name_filter, gbu_filter)
      uri = URI.parse("#{API_BASE_PATH}/admin/organizations")
      query_params = URI.decode_www_form(uri.query || '')
      query_params << ['offset', offset] unless offset.blank?
      query_params << ['limit', limit] unless limit.blank?
      query_params << ['name', name_filter] unless name_filter.blank?
      query_params << ['gbu', gbu_filter] unless gbu_filter.blank?
      uri.query = URI.encode_www_form(query_params)

      get_resource(uri.to_s)
    end

    def self.admin_site(org_id, resource_id)
      # AAA v3 does not have a method to directly retrieve a site by resource id
      # so we have to get all sites for the organization and select it
      sites = get_resource("#{API_BASE_PATH}/admin/organizations/#{org_id}/sites")

      sites[:sites].select { |site| site[:siteId] == resource_id }.first
    end

    def self.authorize(payload, as_service=false)
      # The authorization API implementation is odd in that a response code of 412 is returned to indicate
      # that the service token has expired (normally 401 would be used)
      service_token_expired_code = 412

      begin
        parse PrintosRestClient.post("#{API_BASE_PATH}/authorize", payload, as_service, service_token_expired_code)
      rescue RestClient::Unauthorized
        Printos.config.logger.error('User token is invalid.')
        {}
      end
    end

    def self.system_roles(as_service=false)
      get_resource("#{API_BASE_PATH}/systemroles", as_service)
    end

    def self.system_role_users(org_id, role_id)
      get_resource("#{API_BASE_PATH}/admin/organizations/#{org_id}/users/systemroles/#{role_id}", true)
    end
  end
end
