require 'rest-client'

class PrintosClient

  API_BASE = 'api/aaa/v3'

  USER_PERMISSIONS=%w(admin user hpAdmin).freeze

  def self.set_instance(token)
    @token = token
    ::RequestStore.store[:printos_token] = token
    get_instance
  end

  def self.get_instance
    ::RequestStore.store[:print_os_client] ||= PrintosClient.new(@token)
  end

  def logged_in?
    @token.present?
  end

  def user
    @user ||= Printos::AaaService.user
  end

  def organization
    @org ||= Printos::AaaService.organization
  end

  def site(resource_id)
    @site_resource[resource_id] ||= Printos::AaaService.site(resource_id)
  end

  def sites
    @sites ||= Printos::AaaService.sites
  end

  def printer(resource_id)
    @printer_resource[resource_id] ||= Printos::AaaService.device(resource_id)
  end

  def devices
    org = organization
    site_id = org[:organizationId]

    @devices ||= Printos::AaaService.devices
    device_list = @devices[:devicesWithContext].first[:devices]

    device_list.select { |device| device[:site] && device[:site][:organizationId] == site_id }
  end

  def authorization
    body = {
      requestToken: @token,
      permissions: USER_PERMISSIONS,
      checkedObjectName: 'checkSuluPermissions'
    }

    @authorization ||= Printos::AaaService.authorize(body, true)
  end

  def permissions
    authorization[:permissions]&.map { |p| p[:name] }
  end

  def admin_context
    Printos::AaaService.admin_context
  end

  def admin_device_gbu_organizations(offset, limit, device_gbu_filter, device_type_filter)
    Printos::AaaService.admin_device_gbu_organizations(offset, limit, device_gbu_filter, device_type_filter)
  end

  def admin_organization(org_id)
    Printos::AaaService.admin_organization(org_id)
  end

  def admin_organizations(offset, limit, name_filter, gbu_filter)
    Printos::AaaService.admin_organizations(offset, limit, name_filter, gbu_filter)
  end

  def admin_printer(resource_id)
    @printer_resource[resource_id] ||= Printos::AaaService.admin_device(resource_id)
  end

  def admin_site(org_id, resource_id)
    @site_resource[resource_id] ||= Printos::AaaService.admin_site(org_id, resource_id)
  end

  def update_organization(payload)
    Printos::AaaService.update_organization(payload)
  end

  def create_site(payload)
    Printos::AaaService.create_site(payload)
  end

  def update_site(resource_id, payload)
    Printos::AaaService.update_site(resource_id, payload)
  end

  private
  def initialize(token)
    @token = token
    @site_resource = {}
    @printer_resource = {}
  end
end
